; Generate the files with the observed profiles to carry out the
; Bayesian inversion
pro genprof
	dat = readfits('SP3D20070227_130035.7C.fits', hdr)

; Avoid saturated values
	temp = dat[*,*,0]
	whr = where(temp lt 0., countwrap)
	if (countwrap ne 0) then temp[whr] = temp[whr] + 65536.
	dat[0,0,0] = temp

;  bit shifting for SP data
	bitshft = sxpar(hdr,'SPBSHFT')
;  account for bit shifting of the data by multiplying the appropriate
;  Stokes images by 2
	case(bitshft) of
		3:      dat[*,*,1:2] = 2.*dat[*,*,1:2]
		2:      dat[*,*,3] = 2.*dat[*,*,3]
		1:      dat[*,*,0] = 2.*dat[*,*,0]
	endcase
                
	restore,'wavelength_calibration.idl'
	restore,'local_quiet_continuum.idl'

	sigmaI = 4.13d-3
	sigmaQ = 1.26d-3
	sigmaU = 1.26d-3
	sigmaV = 1.20d-3

; Quiet Sun
	which = 440
	openw,2,'quiet.prof',width=150
	printf,2,'#Quiet Sun profile'
	printf,2,112

	for i = 0, 111 do begin
 		printf, 2, lambda[i], dat[i,which,0]/cont[i], dat[i,which,1]/cont[i], dat[i,which,2]/cont[i], dat[i,which,3]/cont[i],$
 			sigmaI, sigmaQ, sigmaU, sigmaV
	endfor

	close,2


; Sunspot
	which = 205
	openw,2,'sunspot.prof',width=150
	printf,2,'#Sunspot profile'
	printf,2,112
		
	for i = 0, 111 do begin
 		printf, 2, lambda[i], dat[i,which,0]/cont[i], dat[i,which,1]/cont[i], dat[i,which,2]/cont[i], dat[i,which,3]/cont[i],$
 			sigmaI, sigmaQ, sigmaU, sigmaV
	endfor

	close,2
	

	stop
end