! Initialize NLTE computation
! The model is set in hydrostatic equilibrium in order to fill in the 
! required arrays (such as ne, nH, rho, etc)
!
Subroutine NLTE_init(Params, NLTEinput, NLTE, Atom, Atmo)
  Use Param_structure
  Use Atomic_data
  Use NLTE_Atom_structure
  Use NLTE_inp
  Use NLTE_vars
  Use Model_structure
  Use Model_operations
  Use VirtualFiles
  Implicit None
  Type (Parameters) :: Params
  Type (Model) :: Atmo
  Type (NLTE_Atom) :: Atom
  Type (NLTE_input) :: NLTEInput
  Type (NLTE_variables) :: NLTE
  Integer :: ielem
  Logical, Save :: FirstTime=.TRUE.
!
  If (FirstTime) then ! All intializations
     FirstTime=.FALSE.
     Call Read_atom(Atom)
     Call Find_atomic_number
     Atom%Abund=At_abund(Atom%Z)
     Atom%AWgt=At_weight(Atom%Z)
!
     Call Set_hardwired_values ! Need to externalize hardwired values
!
     Call NLTE_allocations ! Allocate NLTE atmospheric arrays
     Call FreqQuad(Atom, NLTEInput) ! Frequency quadrature
     Call AngQuad(NLTE, NLTEInput) ! Angular quadrature
!                               For Virtual Files
     NDEP=NLTE%NDEP
     NRAD=Atom%NLIN+Atom%NCNT
     MQ=MaxVal(Atom%NQ(1:NRAD))
     NMU=NLTEInput%NMu
     Call OpenVirtualFile('OPC',NLTEInput%VirtualFiles)
     Call OpenVirtualFile('PHI',NLTEInput%VirtualFiles)
     Call OpenVirtualFile('JNY',NLTEInput%VirtualFiles)
  End if
! From here on may be needed for repeated calls with different atmospheres
  Call Hydrostatic(Params, NLTE%Atmo) ! To fill in some arrays in Atmo
  Call LTE_pop_2(NLTE, Atom) ! Compute LTE populations
  NLTE%N(:,:)=NLTE%NStar(:,:) ! Populations initial guess
  Call Collisions(NLTE, Atom) ! Collisional rates
  NLTE%C=NLTE%C*NLTEInput%CPER ! Enhance collisions if CPER .gt. 1
  Call FixedRad(NLTE, Atom) ! Continuum transitions with radiation temp
!
  Call BackgroundOpac(NLTE, NLTEInput, Atom) ! Compute background opacities
  Call VoigtProfs(NLTE, NLTEInput, Atom) ! Compute Voigt profiles
!
  Return
!
    contains
      Subroutine NLTE_allocations
        Allocate (NLTE%C(Atom%NK, Atom%NK, NLTE%NDEP))
        NLTE%C(:,:,:)=0.
        Allocate (NLTE%F(Atom%NK, Atom%NK, NLTE%NDEP))
        NLTE%F(:,:,:)=0.
        Allocate (NLTE%W(Atom%NK, Atom%NK, NLTE%NDEP))
        NLTE%W(:,:,:)=0.
        Allocate (NLTE%Nstar(Atom%NK, NLTE%NDEP))
        Allocate (NLTE%N(Atom%NK, NLTE%NDEP))
        Allocate (NLTE%Xnorm(NLTE%NDEP))
        Allocate (NLTE%BPlanck(NLTE%NDEP, Atom%NLIN+Atom%NCNT))
        Allocate (NLTE%XMu(NLTEInput%NMU))
        Allocate (NLTE%WMu(NLTEInput%NMU))
        Allocate (NLTE%WPhi(NLTE%NDEP,Atom%NLIN))
        Allocate (NLTE%Sl(NLTE%NDEP,Atom%NLIN+Atom%NCNT))
        Allocate (NLTE%Source_f(NLTE%NDEP,MaxNFreqs,Atom%NLIN+Atom%NCNT))
!
        Return
      End Subroutine NLTE_allocations
!
      Subroutine Find_atomic_number
        Character (len=256) :: str1, str2
        str1=Atom%element
        Call Tolower(str1)
        Do ielem=1, N_elements
           str2=Atom_char(ielem)
           Call Tolower(str2)
           If (str1 .eq. str2) Atom%Z=ielem
        End do
        Return
      End Subroutine Find_atomic_number
!
      Subroutine Set_hardwired_values
! Some hardwired values (need to externalize)
        NLTE%NDEP=Params%n_points
        NLTEInput%Elim1=1e-4 ! debug. Doesn't work if elim1 .ne. elim2
        NLTEInput%Elim2=1e-4 !   Need to look into it
        NLTEInput%CPER=1.
        NLTEInput%NMU=3
        NLTEInput%QNORM=8
        NLTEInput%VelFree=.True.
        NLTEInput%VirtualFiles=.True.
        NLTEInput%NGacc=.True.
        NLTEInput%MaxIters=400
        NLTEInput%Verbose=1
        If (Params%printout .ge. 2) NLTEInput%Verbose=2
        NLTEInput%ISum=1
        NLTEInput%Write=.False.
        NLTE%Atmo=Atmo ! Use the same grid
        If (NLTEInput%VelFree) NLTE%Atmo%v_los=0. ! Velocity free approximation
        Return
      End Subroutine Set_hardwired_values

End Subroutine NLTE_init
