      LOGICAL FUNCTION NGFUNC(Y,NK,NDEP,N, reset)
!
! Adapted from MULTI
!
!     NG ACCELERATION OF A VECTOR ITERATION SCHEME
!
!     INPUT
!        Y(M)     -ITERATE OF VECTOR ROOT BEING SOUGHT
!        N        =1...5, THE ORDER OF THE ACCELERATION TO BE USED
!        YY(M,N+2)-SCRATCH ARRAY, MUST NOT BE ALTERED AFTER CALL TO NG.
!     OUTPUT
!        NG       =.TRUE. IF AN ACCELERATION HAS BEEN MADE, THEN
!        Y(I)     = ACCELERATED VECTOR
!     RESET
!        CALL NG0 -CLEARS STORAGE OF PREVIOUS ITERATES
!
!     METHOD
!     APPROXIMATE THE ROOT VECTOR IN TERM OF THE PREVIOUS ITERATES Y,YY(1)...
!        ROOT ~ Y + SUM(C(I)*(YY(I) - Y)     AND
!        ROOT ~ YY(1) + SUM(*C(I)*(YY(I+1) - YY(1)).
!     SUBTRACT THESE TWO EXPRESSION.  USE LEAST SQUARES TO DETERMINE C'S.

      DIMENSION Y(NK,NDEP)
!,YY(MK,MDEP,7)
!      DIMENSION YY2(MK*MDEP,7)
      DIMENSION A(7,7),C(7)
      Integer, Save :: NTRY
      Real, Dimension(:,:,:), Allocatable, Save :: YY
      Real, Dimension(:,:), Allocatable, Save :: YY2
      Logical :: reset
      DATA NTRY /0/
!
      NGFUNC=.FALSE.
      If (.not. Allocated(YY)) Allocate(YY(NK, NDEP, 7))
      If (.not. Allocated(YY2)) Allocate(YY2(NK*NDEP, 7))
      If (reset) then
         NTRY=0
         YY(:,:,:)=0.
         YY2(:,:)=0.
         Return
      End if
!
      IF (N.LT.1.OR.N.GT.5) RETURN
      NTRY = NTRY + 1
      YY(1:NK,1:NDEP,NTRY) = Y(1:NK,1:NDEP)
      NGFUNC = .FALSE.
      IF (NTRY.LE.N+1) RETURN
!     ACCELERATION
      A(1:N,1:N)=0.
      C(1:N)=0.
      DO K1=1,NK
         DO K2=1,NDEP
            YY2((K1-1)*NDEP+K2,1:7)=YY(K1,K2,1:7)
         END DO
      END DO
      DO 700 K = 1, NK*NDEP
         WT = 1.D0/(1.d0 + ABS(YY2(K,NTRY)))
         DO 600 I = 1, N
            DY = YY2(K,NTRY-1) - YY2(K,NTRY)
            DI = WT* (DY + YY2(K,NTRY-I) - YY2(K,NTRY-I-1))
            C(I) = C(I) + DI*DY
            DO 500 J = 1, N
               A(I,J) = A(I,J) + DI*(DY+YY2(K,NTRY-J)-YY2(K,NTRY-J-1))
500         END DO
600      END DO
700   END DO
      CALL LLSLV(A,C,N,7)
!     INCLUDE CORRECTION
      NGFUNC = .TRUE.
      DO 1000 I = 1, N
         DO 900 K = 1, NK
            DO K2=1,NDEP
               Y(K,K2) = Y(K,K2) + C(I)*(YY2((K-1)*NDEP+K2,NTRY-I)- &
                    YY2((K-1)*NDEP+K2,NTRY))
            END DO
900      END DO
1000  END DO
!      ENTRY NG0
      NTRY = 0
      RETURN
    END FUNCTION NGFUNC

      SUBROUTINE LLSLV (S, X, N, NR)                                     
!                                                                       
! Adapted from MULTI
!
!     SOLUTION OF THE SYMMETRIC SYSTEM OF LINEAR EQUATIONS              
!           S*X = X                                                     
!     INPUT                                                             
!         S(I,J) = S(J,I)   SYMMETRIC MATRIX ONLY LOWER (J<=I) USED     
!         X(I)   -RIGHT HAND SIDE                                       
!         N      -SIZE OF SYSTEM                                        
!         NR     -ACTUAL FIRST DIMENSION OF S(NR,*)                     
!     OUTPUT                                                            
!         S(I,J),J<=I  = DECOMPOSITION L                                
!         X(I)   -SOLUTION                                              
!     NOTE                                                              
!     THE SYMMETRIC MATRIX IS DECOMPOSED L*LT = S                       
!                                                                       
      REAL S (NR, NR) , X (NR) , SUM                                  
      INTEGER I, J, K, N, NR                                             
!                                                                       
!     FACTOR S(I,J) = SUM(K=1,MIN(I,J): S(I,K)*S(J,K))                  
!                         
      DO 1200 I = 1, N                                                   
         DO 1100 J = 1, I                                                
            SUM = S (I, J)                                               
            DO 1000 K = 1, J - 1                                         
               SUM = SUM - S (I, K) * S (J, K)                           
 1000       END DO                                                       
            IF (J.LT.I) THEN                                             
               S (I, J) = SUM / S (J, J)                                 
            ELSE                                                         
               S (I, J) = SQRT (ABS (SUM) )                            
            ENDIF                                                        
 1100    END DO                                                          
 1200 END DO                                                             
      ENTRY LLRESLV (S, X, N, NR)                                        
!                                                                       
!     SOLVE L*LT X = X                                                  
!           L IS SYMMETRIC FACTOR DETERMINED BY A CALL TO LLSLV         
!                                                                       
      DO 2100 I = 1, N                                                   
         SUM = X (I)                                                     
         DO 2000 J = 1, I - 1                                            
            SUM = SUM - S (I, J) * X (J)                                 
 2000    END DO                                                          
         X (I) = SUM / S (I, I)                                          
 2100 END DO                                                             
      DO 2300 I = N, 1, - 1                                              
         SUM = X (I)                                                     
         DO 2200 J = N, I + 1, - 1                                       
            SUM = SUM - S (J, I) * X (J)                                 
 2200    END DO                                                          
         X (I) = SUM / S (I, I)                                          
 2300 END DO                                                             
      RETURN                                                             
      END SUBROUTINE LLSLV                       
