! This subroutine reads the file NLTE_lines with information to match
! the lines in the LINES file to the NLTE transitions in the ATOM file.
! The format is:
!      iline     itran      ratio_l     ratio_u
! where iline is the index of a line in the LINES file, itran is the
! corresponding transition in the ATOM file and ratio_l is the ratio
! of lower level populations of the line divided by the level population
! in the ATOM file. Same for ratio_u and the upper level.
!
! The information is then used to fill in the corresponding fields in the 
! Line structure
!
Subroutine Read_NLTE_lines(Params, NLTEInput, NLTE, Atom, Line)
  Use NLTE_Atom_structure
  Use Line_data_structure
  Use Param_structure
  Use NLTE_vars
  Use NLTE_inp
  Implicit None
  Type (Parameters) :: Params
  Type (Line_data), Dimension (Params%n_lines) :: Line
  Type (NLTE_Atom) :: Atom
  Type (NLTE_input) :: NLTEInput
  Type (NLTE_variables) :: NLTE
  Integer :: fileunit, itran, ntrans, i1, i2, iline, ix
  Real :: ratiol, ratiou
  Character (len=256) :: String
  Logical :: End
!
! Initialize
!
  Line(:)%NLTEtransition=0
  Line(:)%NLTE_nl_ratio=0.
  Line(:)%NLTE_nu_ratio=0.
  Line(:)%NLTEgridsize=1
  Do iline=1, Params%n_lines
     Allocate (Line(iline)%NLTEgrid(1))
     Line(iline)%NLTEgrid(1)=0.
     Allocate (Line(iline)%NLTESource_f(NLTE%NDEP,1))
     Line(iline)%NLTESource_f(NLTE%NDEP,1)=0.
  End do
!
  Call Open_file(fileunit, 'NLTE_lines')
  End=.False.
  ntrans=0.
  Do While (.not. End)
     Call Read_next_nocomment(fileunit, String, End)
     ntrans=ntrans+1
  End do
  Close (fileunit)
  ntrans=ntrans-1
  Call Open_file(fileunit, 'NLTE_lines')
  Do itran=1, ntrans
     Call Read_next_nocomment(fileunit, String, End)
     If (.not.End) then
        Read (String, *) i1, i2, ratiol, ratiou
        Do iline=1, Params%n_lines
           If (Line(iline)%index .eq. i1) then 
              If (Line(iline)%Atomic_number .ne. Atom%Z .or. &
                   Line(iline)%Ion_stage .ne. Atom%ion(Atom%i(i2)) .or. &
                   Abs(Line(iline)%Energy_low -Atom%eV(Atom%i(i2))) .gt. .1) then 
                 
                 Print *,'Mismatch between data in the LINES file and what is'
                 Print *,'indicated by the combination of ATOM and NLTE_lines:'
                 Print *,'Information in LINES: Z, ion, eV'
                 Print *, &
                      Line(iline)%Atomic_number, Line(iline)%Ion_stage, &
                      Line(iline)%Energy_low
                 Print *,'Information in ATOM (transition',i2,'): Z, ion, eV'
                 Print *,Atom%Z, Atom%ion(Atom%i(i2)), Atom%eV(Atom%i(i2))
                 Stop
              End if
              Line(iline)%NLTEtransition=i2
              Line(iline)%NLTE_nl_ratio=ratiol
              Line(iline)%NLTE_nu_ratio=ratiou
              Line(iline)%NLTEgridsize=Atom%NQ(i2)
              If (Allocated(Line(iline)%NLTEgrid)) &
                   Deallocate(Line(iline)%NLTEgrid)
              Allocate(Line(iline)%NLTEgrid(Atom%NQ(i2)))
              If (Allocated(Line(iline)%NLTESource_f)) &
                   Deallocate(Line(iline)%NLTESource_f)
              Allocate(Line(iline)%NLTESource_f(NLTE%NDEP,Atom%NQ(i2)))
              Line(iline)%NLTEgrid=Atom%Q(1:Atom%NQ(i2),i2)*NLTEInput%QNORM* &
                   1.e5/cc*Atom%Alamb(i2) ! NLTE grid in A from line center
              Line(iline)%NLTESource_f(:,:)=NLTE%Source_f(:,1:Atom%NQ(i2),i2) ! cgs units
           End if
        End do
     End if
  End do
  Close (fileunit)
!
  Return
End Subroutine Read_NLTE_lines
  
