!*****************************************************************  
! This vectorizable Voigt function is based on the paper by Hui, *  
! Armstrong and Wray, JQSRT 19, 509 (1977). Errors become        *  
! significant (at the < 1% level) around the knee between the    *  
! Doppler core and the damping wings for 0.0 < a < 0.001. The    *  
! normalization is such that the integral is equal to SQRT(PI).  *  
! If J <> 0 this function returns the dispersion function.       *  
! -------------------------------------------------------------- *  
! Authors: Jack Harvey, Aake Nordlund.                           *  
! Modified by Sami Solanki (1985).                               *  
! Modified by A.D. Wittmann (1986) to include F(a,-v) and F(0,v) *  
! -------------------------------------------------------------- *  
! Last Update: 18-APR-86.                                        *  
!*****************************************************************  
      FUNCTION VOIGT(A,VV,J)
      COMPLEX Z 
      DIMENSION XDWS(28),YDWS(28)   
      DATA A0,A1,A2,A3,A4,A5,A6,B0,B1,B2,B3,B4,B5,B6/   &
      122.607931777104326,214.382388694706425,181.928533092181549,  &
      93.155580458138441,30.180142196210589,5.912626209773153,  &
      .564189583562615,122.60793177387535,352.730625110963558,  &
      457.334478783897737,348.703917719495792,170.354001821091472,  &
      53.992906912940207,10.479857114260399/
      DATA XDWS/.1,.2,.3,.4,.5,.6,.7,.8,.9,1.,1.2,1.4,1.6,1.8,2.,   &
      3.,4.,5.,6.,7.,8.,9.,10.,12.,14.,16.,18.,20./,YDWS/  &
      9.9335991E-02,1.9475104E-01,2.8263167E-01,3.5994348E-01, &
      4.2443639E-01,4.7476321E-01,5.1050407E-01,5.3210169E-01, &
      5.4072434E-01,5.3807950E-01,5.0727350E-01,4.5650724E-01, &
      3.9993989E-01,3.4677279E-01,3.0134040E-01,1.7827103E-01, &
      1.2934799E-01,1.0213407E-01,8.4542692E-02,7.2180972E-02, &
      6.3000202E-02,5.5905048E-02,5.0253846E-02,4.1812878E-02, &
      3.5806101E-02,3.1311397E-02,2.7820844E-02,2.5031367E-02/ 

 
      V=ABS(VV) 
      IF(A.NE.0) GOTO 1 
      IF(J.NE.0) GOTO 3 
      VOIGT=EXP(-V*V)   
      RETURN
   3  IF(V.GT.XDWS(1)) GOTO 4   
      D=V*(1.-.66666667*V*V)
      GOTO 8
   4  IF(V.GT.XDWS(28)) GOTO 5  
      K=27  
      DO 7 I=2,27   
      IF(XDWS(I).LT.V) GOTO 7   
      K=I   
      GOTO 6
   7  CONTINUE  
   6  KK=K-1
      KKK=K+1   
      D1=V-XDWS(KK) 
      D2=V-XDWS(K)  
      D3=V-XDWS(KKK)
      D12=XDWS(KK)-XDWS(K)  
      D13=XDWS(KK)-XDWS(KKK)
      D23=XDWS(K)-XDWS(KKK) 
      D=YDWS(KK)*D2*D3/(D12*D13)-YDWS(K)*D1*D3/(D12*D23)+YDWS(KKK)* &
      D1*D2/(D13*D23)   
      GOTO 8
   5  Y=.5/V
      D=Y*(1.+Y/V)  
   8  VOIGT=5.641895836E-1*D
   9  IF(VV.LT.0.) VOIGT=-VOIGT 
      RETURN
   1  Z=CMPLX(A,-V) 
      Z=((((((A6*Z+A5)*Z+A4)*Z+A3)*Z+A2)*Z+A1)*Z+A0)/   &
      (((((((Z+B6)*Z+B5)*Z+B4)*Z+B3)*Z+B2)*Z+B1)*Z+B0)  
      IF(J.NE.0) GOTO 2 
      VOIGT=REAL(Z)
      RETURN
   2  VOIGT=.5*AIMAG(Z) 

      GOTO 9
      END 
  
!*****************************************************************  
! This vectorizable Voigt function is based on the paper by Hui, *  
! Armstrong and Wray, JQSRT 19, 509 (1977). Errors become        *  
! significant (at the < 1% level) around the knee between the    *  
! Doppler core and the damping wings for 0.0 < a < 0.001. The    *  
! normalization is such that the integral is equal to SQRT(PI).  *  
! If J <> 0 this function returns the dispersion function.       *  
! -------------------------------------------------------------- *  
! Authors: Jack Harvey, Aake Nordlund.                           *   
! Modified by Sami Solanki (1985).                               *  
! Modified by A.D. Wittmann (1986) to include F(a,-v) and F(0,v) *
! convertida en rutina por Basilio Ruiz (1993)                   *
! -------------------------------------------------------------- *  
! Last Update: 22-jun 93.                                        *  
!*****************************************************************  
      subroutine VOIGT2(A,VV,H,F)
      Implicit Real*8 (a-h,o-z)
      COMPLEX Z 
      DIMENSION XDWS(28),YDWS(28)
      DATA A0,A1,A2,A3,A4,A5,A6,B0,B1,B2,B3,B4,B5,B6/   &
      122.607931777104326,214.382388694706425,181.928533092181549,  &
      93.155580458138441,30.180142196210589,5.912626209773153,  &
      .564189583562615,122.60793177387535,352.730625110963558,  &
      457.334478783897737,348.703917719495792,170.354001821091472, &
      53.992906912940207,10.479857114260399/
      DATA XDWS/.1,.2,.3,.4,.5,.6,.7,.8,.9,1.,1.2,1.4,1.6,1.8,2.,   &
      3.,4.,5.,6.,7.,8.,9.,10.,12.,14.,16.,18.,20./,YDWS/  &
      9.9335991E-02,1.9475104E-01,2.8263167E-01,3.5994348E-01,  &
      4.2443639E-01,4.7476321E-01,5.1050407E-01,5.3210169E-01,  &
      5.4072434E-01,5.3807950E-01,5.0727350E-01,4.5650724E-01,  &
      3.9993989E-01,3.4677279E-01,3.0134040E-01,1.7827103E-01,  &
      1.2934799E-01,1.0213407E-01,8.4542692E-02,7.2180972E-02,  &
      6.3000202E-02,5.5905048E-02,5.0253846E-02,4.1812878E-02,  &
      3.5806101E-02,3.1311397E-02,2.7820844E-02,2.5031367E-02/



	ivsigno=1
        if(vv.lt.0)ivsigno=-1
        v=ivsigno*vv

        IF(A.eq.0)then 
	  v2=V*V       				
          H=EXP(-v2)   
      				
          IF(V.GT.XDWS(1)) GOTO 4   
          D=V*(1.-.66666667*v2)
          GOTO 8
   4      IF(V.GT.XDWS(28)) GOTO 5  
          K=27  
          DO 7 I=2,27   
             IF(XDWS(I).LT.V) GOTO 7   
             K=I   
             GOTO 6
   7      CONTINUE  
   6      KK=K-1
          KKK=K+1   
          D1=V-XDWS(KK) 
          D2=V-XDWS(K)  
          D3=V-XDWS(KKK)
          D12=XDWS(KK)-XDWS(K)  
          D13=XDWS(KK)-XDWS(KKK)
          D23=XDWS(K)-XDWS(KKK) 
          D=YDWS(KK)*D2*D3/(D12*D13)-YDWS(K)*D1*D3/(D12*D23)+YDWS(KKK)* &
          D1*D2/(D13*D23)   
          GOTO 8
   5      Y=.5/V
          D=Y*(1.+Y/V)  
   8      F=ivsigno*5.641895836E-1*D

! si el damping no es nulo
	else

           Z=CMPLX(A,-V) 
           Z=((((((A6*Z+A5)*Z+A4)*Z+A3)*Z+A2)*Z+A1)*Z+A0)/   &
             (((((((Z+B6)*Z+B5)*Z+B4)*Z+B3)*Z+B2)*Z+B1)*Z+B0)
           
           H=REAL(Z) 
           F=.5*ivsigno*AIMAG(Z) 
             
	end if

	return
        END   



!***********************************************************************
!      SUBROUTINE HUMLICEK (NX,X,Y, PRBFCT)
      subroutine voigt3 (nwave, a,vv,H,F)
!                                                                      *
!     complex probability function for complex argument Z=X+iY         *
!     real part = voigt function K(x,y)                                *
!                                                                      *
!     source:   j. humlicek, JQSRT 27, 437, 1982                       *
!                                                                      *
!     parameters:                                                      *
!      NX     number of grid points = NX+1                         in  *
!      X      array of grid points                                 in  *
!      Y      Voigt function parameter, ratio of lorentz/doppler   in  *
!      PRBFCT complex array of function values                     out *
!                                                                      * 
!     the stated accuracy is claimed to be 1.0E-04 by the author.      *
!     r h norton has checked the accuracy by comparing values          *
!     computed using a program written by b.h.armstrong, and           *
!     the accuracy claim seems to be warranted.                        *
!                                                                      *
!*************************************************************fgs 12/91*
      REAL      X(0:nwave-1), Y, S
      COMPLEX   T, U, PRBFCT(0:nwave-1)
      COMPLEX   APPROX1, APPROX2, APPROX3, APPROX4

      Real a,vv(nwave),h(nwave),f(nwave)
      Integer nwave
      
      APPROX1(T)   = (T * .5641896) / (.5 + (T * T))
      APPROX2(T,U) = (T * (1.410474 + U*.5641896))/ (.75 + (U *(3.+U)))
      APPROX3(T)   = ( 16.4955 + T * (20.20933 + T * (11.96482 + &
           T * (3.778987 + 0.5642236*T)))) &
           / ( 16.4955 + T * (38.82363 + T * &
           (39.27121 + T * (21.69274 + T * (6.699398 + T)))))
     
      APPROX4(T,U) = (T * (36183.31 - U * (3321.99 - U * (1540.787 - U &
               *(219.031 - U *(35.7668 - U *(1.320522 - U * .56419)))))) &
             / (32066.6 - U * (24322.8 - U * (9022.23 - U * (2186.18  &
                - U * (364.219 - U * (61.5704 - U * (1.84144 - U))))))))
!     ==================================================================

      NX=nwave-1
      X(0:nwave-1)=vv(1:nwave)
      Y=a

      F(1:nwave)=1
      Do i=0, nwave-1
        If (X(i) .lt. 0) F(i+1)=-1
        X(i)=Abs(X(i))
      End do

!     ==================================================================
             
      IF (Y.GT.15.) THEN
!        ---------------------------------------------------------------
!        all points are in region I
         DO 100, I=0,NX
           T         = CMPLX(Y,-X(I))
 100       PRBFCT(I) = APPROX1(T)
!        ---------------------------------------------------------------
      ELSE IF (Y.LT.15. .AND. Y.GE.5.5) THEN
!        ---------------------------------------------------------------
!        points are in region I or region II
         DO 200, I=0,NX
           T  = CMPLX(Y,-X(I))
           S  = ABS(X(I)) + Y
           IF (S .GE. 15.) THEN
              PRBFCT(I) = APPROX1(T)
           ELSE
              U      = T * T
              PRBFCT(I) = APPROX2(T,U)
           END IF
 200       CONTINUE
!       ----------------------------------------------------------------
      ELSE IF (Y.LT.5.5 .AND. Y.GT.0.75) THEN
!        ---------------------------------------------------------------
         DO 300, I=0,NX
           T  = CMPLX(Y,-X(I))
           S  = ABS(X(I)) + Y
           IF (S .GE. 15.) THEN
              PRBFCT(I) = APPROX1(T)
           ELSE IF (S.LT.5.5) THEN
              PRBFCT(I) = APPROX3(T)
           ELSE
              U      = T * T
              PRBFCT(I) = APPROX2(T,U)
           END IF
 300       CONTINUE
!       ----------------------------------------------------------------
      ELSE
!        ---------------------------------------------------------------
         DO 400, I=0,NX
           T  = CMPLX(Y,-X(I))
           AX = ABS(X(I))
           S  = AX + Y
           IF (S .GE. 15.) THEN
!              region I
               PRBFCT(I)= APPROX1(T)
           ELSE IF (S.LT.15.0 .AND. S.GE.5.5) THEN
!              region II
               U = T * T
               PRBFCT(I)= APPROX2(T,U)
           ELSE IF (S.LT.5.5 .AND. Y.GE.(0.195*AX-0.176)) THEN
!             region III
              PRBFCT(I)= APPROX3(T)
           ELSE
!             region IV
              U  = T * T
              PRBFCT(I)= CEXP(U) - APPROX4(T,U)
           END IF
 400       CONTINUE
!        ---------------------------------------------------------------
      END IF
!     ==================================================================
      IF (Y .EQ. 0.0) THEN
         DO 20, I=0,NX
20         PRBFCT(I) = CMPLX(EXP(-X(I)**2), AIMAG(PRBFCT(I)))
         END IF
!     ==================================================================


         h(1:nwave)=real(prbfct(0:nwave-1))
         f(1:nwave)=.5*f(1:nwave)*imag(prbfct(0:nwave-1))

      RETURN
      END


!--------------------------------------------------------------
! Generates Voigt and anomalous dispersion profiles using a different damping for each frequency
! point
! See Humlicek (1982) JQSRT 27, 437
!--------------------------------------------------------------
	subroutine voigt4 (nwave, a,vv,H,F)
	Real a,vv(nwave),h(nwave),f(nwave)
   Integer nwave
	complex :: w4, z, t, u, v4
	real :: s
	integer :: i, n
                		
		if (a < 1.d-3) a = 0.d0

		do i = 1, nwave
			z = cmplx(vv(i),a)
			t = cmplx(a, -vv(i))
			s = abs(vv(i)) + a
			u = t*t
		
			if (s >= 15.d0) then
				w4 = t * 0.5641896d0 / (0.5d0+u)
				H(i) = dble(w4)
				F(i) = aimag(w4)
				cycle
			endif
			
			if (s >= 5.5d0) then
				w4 = t*(1.410474d0+u*0.5641896d0)/(0.75d0+u*(3.d0+u))
				H(i) = dble(w4)
				F(i) = aimag(w4)
				cycle
			endif

			if (a >= 0.195d0*abs(vv(i))-0.176d0) then
				w4 = (16.4955d0+t*(20.20933d0+t*(11.96482d0+t*(3.778987d0+t*0.5642236d0)))) / &
					(16.4955d0+t*(38.82363d0+t*(39.27121d0+t*(21.69274d0+t*(6.699398d0+t)))))
				H(i) = dble(w4)
				F(i) = aimag(w4)
				cycle
			endif

			w4 = t*(36183.31d0-u*(3321.9905d0-u*(1540.787d0-u*(219.0313d0-u*(35.76683d0-&
					u*(1.320522d0-u*0.56419d0))))))
			v4 = (32066.6d0-u*(24322.84d0-u*(9022.228d0-u*(2186.181d0-u*(364.2191d0-&
					u*(61.57037d0-u*(1.841439d0-u)))))))
			w4 = cexp(u) - w4/v4

			H(i) = dble(w4)
			F(i) = aimag(w4)
			
		enddo
	 end subroutine voigt4

!***********************************************************************
      subroutine voigt5 (nwave, a,vv,H,F)
      integer :: nwave
      Real a,vv(nwave),h(nwave),f(nwave), v(nwave)
      complex :: z(nwave)
!*****************************************************************
! This vectorizable Voigt function is based on the paper by Hui, *
! Armstrong and Wray, JQSRT 19, 509 (1977). Errors become        *
! significant (at the < 1% level) around the knee between the    *
! Doppler core and the damping wings for 0.0 < a < 0.001. The    *
! normalization is such that the integral is equal to SQRT(PI).  *
! If J <> 0 this function returns the dispersion function.       *
! -------------------------------------------------------------- *
! Authors: Jack Harvey, Aake Nordlund.                           *
! Modified by Sami Solanki (1985).                               *
! Modified by A.D. Wittmann (1986) to include F(a,-v) and F(0,v) *
! convertida en rutina por Basilio Ruiz (1993)                   *
! -------------------------------------------------------------- *
!*****************************************************************
			A0=122.607931777104326d0
      	A1=214.382388694706425d0
      	A2=181.928533092181549d0
      	A3=93.155580458138441d0
      	A4=30.180142196210589d0
      	A5=5.912626209773153d0
      	A6=.564189583562615d0
      	B0=122.60793177387535d0
      	B1=352.730625110963558d0
      	B2=457.334478783897737d0
      	B3=348.703917719495792d0
      	B4=170.354001821091472d0
      	B5=53.992906912940207d0
      	B6=10.479857114260399d0

      	v = abs(vv)
      	z = cmplx(a, -v)
      	z = ((((((A6*Z+A5)*Z+A4)*Z+A3)*Z+A2)*Z+A1)*Z+A0)/&
            (((((((Z+B6)*Z+B5)*Z+B4)*Z+B3)*Z+B2)*Z+B1)*Z+B0)
         H = real(Z)
         F = vv/abs(vv)*aimag(Z)
      end subroutine voigt5