Module Model_structure
!
  Type Model
! temp is in K, v_los and v_mic are in cm/s, el_p and gas_p are in dyn/cm^2, 
! rho is in g/cm^3, b_str is in gauss, b_inc and b_azi are in degrees
! ne, nH, nHplus are electron, neutral H and HII number densities in cm^-3
! nHminus and nH2 are the number densities in cm^-3 of the negative H ion and 
! molecular Hydrogen respectively
! v_mac is in cm/s and stray is dimensionless (0 < stray < 1).
     real, dimension(:), pointer :: z_scale, ltau_500, temp, gas_p, rho, el_p
     real, dimension(:), pointer :: v_los, v_mic, b_str, b_inc, b_azi
     real, dimension(:), pointer :: local_inc, local_azi
     real, dimension(:), pointer :: ne, nH, nHplus
     real, dimension(:), pointer :: nHminus, nH2
     real :: v_mac, stray, expansion
  End Type Model
!
End Module Model_structure
!
!
!
Module Model_operations
!
  Interface assignment (=)
     Module procedure Model_assign
  End interface
!
Contains
  Subroutine Model_assign(A,B)
    Use Model_structure
    Type (Model), intent(out) :: A
    Type (Model), intent(in) :: B
    Integer :: status, npoints
    !
    If (.NOT.Associated(A%z_scale)) then
       npoints=size(B%z_scale)
       Call Allocate_Model(A, npoints)
    End if
    If (size(A%temp) .eq. 0) then
       print *,'Error in model_struct! Model is allocated, but has zero size'
       stop
    End if
    A%v_mac=B%v_mac
    A%stray=B%stray
    A%expansion=B%expansion
    A%z_scale=B%z_scale
    A%ltau_500=B%ltau_500
    A%temp=B%temp
    A%gas_p=B%gas_p
    A%rho=B%rho
    A%el_p=B%el_p
    A%v_los=B%v_los
    A%v_mic=B%v_mic
    A%B_str=B%B_str
    A%B_inc=B%B_inc
    A%B_azi=B%B_azi
    A%ne=B%ne
    A%nH=B%nH
    A%nHplus=B%nHplus
    A%nHminus=B%nHminus
    A%nH2=B%nH2
    A%local_inc=B%local_inc
    A%local_azi=B%local_azi
  End subroutine Model_assign
!
End module Model_operations
