! This subroutine takes a structure Atom and computes the fixed
! radiative rates (Radiation temperatures).
!
Subroutine FixedRad(NLTE, Atom)
  Use NLTE_Atom_structure
  Use NLTE_vars
  Use Param_structure
  Implicit None
  Type (NLTE_Atom) :: Atom
  Type (NLTE_variables) :: NLTE
  Integer :: K, i, j, itran
  Real :: T, TR, Delt, XXI, CLAM, CA0, RIJK, RJIK, STIMD, EX, X2, &
       Rint, Rint1, HKTT
!
  Do itran=1, Atom%NFIX
     j=Atom%JFIX(itran)
     i=Atom%IFIX(itran)
     XXI=Atom%eV(J)-Atom%eV(I)
     CLAM=hh*cc/ee/XXI
     Do K=1, NLTE%NDEP
        T=NLTE%Atmo%Temp(K)
        DELT=-1.0
        If (K .ne. NLTE%NDEP) DELT=NLTE%Atmo%Temp(K)-NLTE%Atmo%Temp(K+1)
        TR=T
        If (Atom%ITRad(itran) .eq. 2) then
           If (DELT .ge. 0.) TR=Atom%TRad(itran)
           If (DELT .lt. 0. .and. Atom%TRad(itran) .gt. T) &
                TR=Atom%TRad(itran)
        Else if (Atom%ITRad(itran) .eq. 3 .and. Atom%TRad(itran) .lt. T &
             .and. DELT .ge. 0.) then
           TR=Atom%TRad(itran)
        Endif
!
        If (Atom%IPho(itran) .eq. 1) then
           CA0=8.*Pi*cc/CLAM/CLAM/CLAM*Atom%A0(itran)
           RIJK=CA0*Rint(ee/bk/TR*XXI)
           RJIK=RIJK
           If (TR .ne. T) RJIK=CA0*RINT(ee/bk/T*XXI)
           HKTT=ee/bk*XXI/(T*TR)
           STIMD=RINT1(T,TR,HKTT)*CA0
           RJIK=(RJIK+STIMD)*NLTE%NSTAR(i,K)/NLTE%NSTAR(j,K)
        Else
           EX=EXP(-ee/bk/TR*XXI)
           X2=(EE*XXI/HH)**2
           RIJK=7.421e-22*Atom%A0(itran)*X2*EX/(1.-EX)
           RJIK=RIJK
           If (TR .ne. T) then
              EX=EXP(-ee/bk/T*XXI)
              RJIK=7.421e-22*Atom%A0(itran)*X2*EX/(1.-EX)
           End if
           RJIK=RJIK*Atom%g(i)/(EX*Atom%g(j))
        End if
        NLTE%F(i,j,K)=RIJK
        NLTE%F(j,i,K)=RJIK
     End do
  End do
  Return
!
End Subroutine FixedRad

Function Rint(X)
!
!  EVALUATES THE INTEGRAL NEEDED IN CALCULATING THE FIXED RAD. RATES
!  BY FINDING THE SUM OF THE EXPONENTIAL INTEGRALS.
!
  Implicit None
  Real :: Sum, Exint1, X, XX, DSUM, Rint
  Integer :: I
!
  SUM=Exint1(X)
  If (SUM .eq. 0.0) then
     Rint=0.
     Return
  Else
     Do I=2,999
        XX=I*X
        DSUM=Exint1(XX)
        SUM=SUM+DSUM
        If (DSUM/SUM .le. 1.E-5) then
           Rint=SUM
           Return
        End if
     End do
  End if
  Rint=SUM
  Return
End Function Rint

Function Rint1(TE, TR, HKTT)
  Implicit None
  Real :: TE, TR, HKTT, SUM, Exint1, XX, DSUM, Rint1
  Integer :: i
!
  SUM=Exint1((TE+TR)*HKTT)
  If (SUM .eq. 0.0) then
     Rint1=SUM
     Return
  Else
     Do i=2, 999
        XX=(i*TE+TR)*HKTT
        DSUM=EXINT1(XX)
        SUM=SUM+DSUM
        If (DSUM/SUM .le. 1.e-5) then
           Rint1=SUM
           Return
        End if
     End do
  End if
  Rint1=SUM
  Return
End Function Rint1

Function Exint1(X)
!
!  CALCULATES THE EXPONENTIAL INTEGRAL FUNCTION E1(X)
!
!  REFERENCE: HANDBOOK OF MATHEMATICAL FUNCTIONS,
!  NBS APPLIED MATHEMATICS SERIES 55 (1964) (ED. ABRAMOVITZ
!  AND STEGUN), EQUATIONS 5.1.53 AND 5.1.54.
!
  Implicit None
  Real, Dimension(6) :: A
  Real, Dimension(4) :: B
  Real :: Exint1, S, Y, X
  Integer :: i
  Data A/-0.57721566,0.99999193,-0.24991055,0.05519968, &
       -0.00976004,0.00107857/
  Data B/2.334733,0.250621,3.330657,1.681534/
!
  If (X .gt. 80.0) then
     Exint1=0.
  Else if (X .gt. 1.0) then
     S=X*X+B(1)*X+B(2)
     S=S/(X*X+B(3)*X+B(4))
     Exint1=S*EXP(-X)/X
  Else
     S=0.0
     Y=1.0
     Do i=1,6
        S=S+A(I)*Y
        Y=Y*X
     End do
     Exint1=S-LOG(X)
  End if
  Return
End Function Exint1

