Module VirtualFiles
  Use Param_structure ! Just to get the RealBytes parameter
  Integer :: ipointopc, ipointphi ! Virtual file pointers. Initialized in
                                  !   OpenVirtualFile or RewindVirtualFile
  Integer :: NDEP, NRAD, MQ, NMU ! Initialized in NLTE_init
  Integer :: lJNY, lPHI, lOPC ! Unit numbers
  Real, Dimension(:), Allocatable :: DJNY,DPHI,DXCONT,DSCAT,DSC ! Storage
!
  Contains
    Subroutine OpenVirtualFile(Filename, Virtual)
      Implicit None
      Character (Len=*) :: Filename
      Character (Len=3) :: File
      Logical :: Virtual
      Integer :: IRECW, iunit
!                      
      File=Filename(1:3)
      IRECW=NDEP*NRAD*MQ
      If (File .ne. 'JNY' .and. File .ne. 'OPC' .and. File .ne. 'PHI') then
         Print *,'Trying to open wrong file:',Filename
         Print *,'in forward/NLTE/virtualfiles.f90'
         Stop
      End if
      If (Virtual) then ! Memory storage
         If (File .eq. 'JNY') then
            lJNY=-10
            Allocate (DJNY(NDEP*NRAD*MQ))
         Else if (File .eq. 'OPC') then
            lOPC=-20
            ipointopc=1
            Allocate (DXCONT(NDEP*NRAD*MQ))
            Allocate (DSCAT(NDEP*NRAD*MQ))
            ALLOCATE (DSC(NDEP*NRAD*MQ))
         Else if (File .eq. 'PHI') then
            lPHI=-30
            ipointphi=1
            Allocate (DPHI(NDEP*NRAD*MQ*NMU))
         End if
         Return
      Else
         If (File .eq. 'JNY') then ! Unformatted direct
            Call Open_file_direct(iunit, File, IRECW)
            lJNY=iunit
         Else if (File .eq. 'PHI') then ! Formatted sequential
            Call Open_file(iunit, File)
            lPHI=iunit
         Else if (File .eq. 'OPC') then ! Formatted sequential
            Call Open_file(iunit, File)
            lOPC=iunit
         End if
      End if
      Return
    End Subroutine OpenVirtualFile
!
    Subroutine CloseVirtualFile(iunit)
      Implicit None
      Character (len=3) :: File
      Integer :: iunit
      If (File .ne. 'JNY' .and. File .ne. 'OPC' .and. File .ne. 'PHI') then
         Print *,'Trying to close wrong file:',File
         Print *,'in forward/NLTE/virtualfiles.f90'
         Stop
      End if
      If (File .eq. 'JNY') then
         iunit=lJNY
      Else if (File .eq. 'OPC') then
         iunit=lOPC
      Else if (File .eq. 'PHI') then
         iunit=lPHI
      End if
      If (iunit .gt. 0) then ! Disk file
         Close (iunit)
      Else ! Memory file
         If (iunit .eq. -10) Deallocate (DJNY)
         If (iunit .eq. -20) then
            ipointopc=1
            Deallocate (DXCONT)
            Deallocate (DSCAT)
            Deallocate (DSC)
         End If
         If (iunit .eq. -30) Then
            ipointphi=1
            Deallocate (DPHI)
         End if
      End if
      Return
    End Subroutine CloseVirtualFile
!
    Subroutine RewindVirtualFile(File)
      Implicit None
      Character (len=3) :: File
      Integer :: iunit
      If (File .ne. 'JNY' .and. File .ne. 'OPC' .and. File .ne. 'PHI') then
         Print *,'Trying to rewind wrong file:',File
         Print *,'in forward/NLTE/virtualfiles.f90'
         Stop
      End if
      If (File .eq. 'JNY') then
         iunit=lJNY
      Else if (File .eq. 'OPC') then
         iunit=lOPC
      Else if (File .eq. 'PHI') then
         iunit=lPHI
      End if
      If (iunit .gt. 0) then
         Rewind iunit
      Else
         If (iunit .eq. -20) ipointopc=1
         If (iunit .eq. -30) ipointphi=1
      End if
      Return
    End Subroutine RewindVirtualFile
!
    Subroutine ReadX(XCONT, SCAT, SC)
      Implicit None
      Integer :: initial, final, ND, iunit
      Real, Dimension(*) :: XCONT, SCAT, SC

      iunit=lOPC
      If (iunit .gt. 0) then
         Read (iunit, *) XCONT(1:NDEP), SCAT(1:NDEP), SC(1:NDEP)
      Else
         initial=(ipointopc-1)*NDEP+1
         final=ipointopc*NDEP
         XCONT(1:NDEP)=DXCONT(initial:final)
         SCAT(1:NDEP)=DSCAT(initial:final)
         SC(1:NDEP)=DSC(initial:final)
         ipointopc=ipointopc+1
      End if
      Return
    End Subroutine ReadX
!
    Subroutine WriteX(XCONT, SCAT, SC)
      Implicit None
      Integer :: initial, final, ND, iunit
      Real, Dimension(*) :: XCONT, SCAT, SC

      iunit=lOPC
      If (iunit .gt. 0) then
         Write (iunit, *) XCONT(1:NDEP), SCAT(1:NDEP), SC(1:NDEP)
      Else
         initial=(ipointopc-1)*NDEP+1
         final=ipointopc*NDEP
         DXCONT(initial:final)=XCONT(1:NDEP)
         DSCAT(initial:final)=SCAT(1:NDEP)
         DSC(initial:final)=SC(1:NDEP)
         ipointopc=ipointopc+1
      End if
      Return
    End Subroutine WriteX
!
    Subroutine ReadJ(irec, JNY)
      Implicit None
      Integer :: initial, final, ND, irec, iunit
      Real, Dimension(*) :: JNY

      iunit=lJNY
      If (iunit .gt. 0) then
         Read (iunit, REC=irec) JNY(1:NDEP)
      Else
         initial=(irec-1)*NDEP+1
         final=irec*NDEP
         JNY(1:NDEP)=DJNY(initial:final)
      End if
      Return
    End Subroutine ReadJ
!
    Subroutine WriteJ(irec, JNY)
      Implicit None
      Integer :: initial, final, ND, irec, iunit
      Real, Dimension(*) :: JNY(1:NDEP)

      iunit=lJNY
      If (iunit .gt. 0) then
         Write (iunit, REC=irec) JNY
      Else
         initial=(irec-1)*NDEP+1
         final=irec*NDEP
         DJNY(initial:final)=JNY(1:NDEP)
      End if
      Return
    End Subroutine WriteJ
!
    Subroutine ReadP(PHI)
      Implicit None
      Integer :: initial, final, ND, iunit
      Real, Dimension(*) :: PHI

      iunit=lPHI
      If (iunit .gt. 0) then
         Read (iunit, *) PHI(1:NDEP)
      Else
         initial=(ipointphi-1)*NDEP+1
         final=ipointphi*NDEP
         PHI(1:NDEP)=DPHI(initial:final)
         ipointphi=ipointphi+1
      End if
      Return
    End Subroutine ReadP
!
    Subroutine WriteP(PHI)
      Implicit None
      Integer :: initial, final, ND, iunit
      Real, Dimension(*) :: PHI

      iunit=lPHI
      If (iunit .gt. 0) then
         Write (iunit, *) PHI(1:NDEP)
      Else
         initial=(ipointphi-1)*NDEP+1
         final=ipointphi*NDEP
         DPHI(initial:final)=PHI(1:NDEP)
         ipointphi=ipointphi+1
      End if
      Return
    End Subroutine WriteP
!
End Module VirtualFiles
