!	...............................................................
!	gasc calculates the gas pressure and the partial pressures
!     of the different elements from the temperature and the electron
!     pressure


      subroutine gasc(t,pe,pg,pp)
      use Param_structure
      use Atomic_data
      parameter (ncontr=28)
      real cmol(91),alfai(ncontr),chi, &
       u0(ncontr),u1(ncontr),u2(ncontr)
      real du0,du1,du2,dcmol(91)
      real pp(*),p(28), tar(1), anear(1), onear(1)
      real g2ar(1), g3ar(1), aar(1), bar(1)
! Interface for saha
      Interface
         Function Saha(npoints, T, Ne, U1, U2, E_ioniz)
           Real, dimension (npoints) :: T, Ne, U1, U2, Saha
           Real :: E_ioniz
           Integer :: npoints
         End Function Saha
      End Interface
!
      tar(1)=t
      onear(1)=1.
      theta=5040./t
      call molecb(theta,cmol,dcmol)
      g4=pe*10.**(cmol(1))
      g5=pe*10.**(cmol(2))
!	ahora calculo los niveles u0,u1,u2 y sus derivadas
6     do i=1,ncontr
         Call Partition_f(i,t,u0(i),u1(i),u2(i),du0,du1,du2)
      End do
      
      ane=pe/bk/t
      anear(1)=ane
      chi=At_ioniz1(1)*eV_to_cgs
      g2ar=1./Saha(1,tar,anear,u0(1),u1(1),chi)   ! p(h+)/p(h)
      g2=g2ar(1)
      If (g2 .le. 1.e-37) g2=0.
      chi=0.754*eV_to_cgs
      g3ar=Saha(1,tar,anear,onear,u0(1),chi)     ! p(h-)/p(h)
      g3=g3ar(1)
      If (g3 .le. 1.e-37) g3=0.
      pp(10)=g3
      g1=0.
      do 1 i=2,ncontr
        chi=At_ioniz1(i)*eV_to_cgs
        aar=1./Saha(1,tar,anear,u0(i),u1(i),chi)
        a=aar(1)
        If (a .le. 1.e-37) a=0.
        chi=At_ioniz2(i)*eV_to_cgs
        bar=1./Saha(1,tar,anear,u1(i),u2(i),chi)
        b=bar(1)
        If (b .le. 1.e-37) b=0.
	c=1.+a*(1.+b) !Nmero total partido por nmero de neutros
        p(i)=(10.**(At_abund(i)-12.))/c   ! p/ph' for neutral he,li, ...
1	g1=g1+p(i)*a*(1.+2.*b)


        pp(2)=p(2)
        pp(3)=p(6)
        pp(4)=p(11)
        pp(5)=p(12)
 
        a=1.+g2+g3
        b=2.*(1.+g2/g5*g4)
        c=g5
        d=g2-g3
        e=g2/g5*g4
        c1=c*b**2+a*d*b-e*a**2
        c2=2.*a*e-d*b+a*b*g1	
        c3=-(e+b*g1)
        f1=0.5*c2/c1

        
! Hctor: Check that f1 won't go to infinity at high temperatures

        if (abs(f1).lt.abs(1e3*c3/c1)) then
          f1=-f1+sign(1.,c1)*sqrt(f1**2-c3/c1)
        else
          f1=f1*(-1+sign(1.,c1))-sign(1.,c1)*.5*c3/c1/f1
        end if        

        f5=(1.-a*f1)/b
        f4=e*f5
        f3=g3*f1
        f2=g2*f1
        fe=f2-f3+f4+g1
        If (fe .eq. 0) fe=Max(Abs(f2),Abs(f3),Abs(f4),Abs(g1))/1000.
	phtot=pe/fe

        if(f5.gt.1.e-4) goto 2
          const6=g5/pe*f1**2
          const7=f2-f3+g1

	  do 3 i=1,5
      	       f5=phtot*const6
	       f4=e*f5
	       fe=const7+f4
	       phtot=pe/fe
3	  continue
      
2       pg=pe*(1.+(f1+f2+f3+f4+f5+0.1014)/fe)
        pp(1)=f1   ! p(h)/p(h')
        pp(6)=f2   ! p(h+)/p(h')	
        pp(7)=f5   ! p(h2)/p(h')
        pp(8)=fe   ! pe/p(h')
        pp(9)=pe/(1.38054e-16*t) ! n(e)=pe/kt

      return
End subroutine gasc
!
Subroutine Molecb(Theta, Cmol, Dcmol)
  Real :: Theta
  Real, dimension(91) :: Cmol, Dcmol
!
      CMOL(1)=-11.206998+THETA*(2.7942767+THETA*(7.9196803E-2-THETA*2.4790744E-2)) ! H2+
      CMOL(2)=-12.533505+THETA*(4.9251644+THETA*(-5.6191273E-2+THETA*3.2687661E-3)) ! H2
      dTheta=(-Theta*Theta)/5040.

      dcmol(1)=dTheta*(2.7942767+Theta*(2*7.9196803e-2-Theta*3*2.4790744e-2))
      dcmol(2)=dTheta*(4.9251644+Theta*(-2*5.6191273e-2-Theta*3*3.2687661e-3))
  RETURN
End Subroutine Molecb
!
!	...............................................................
!	gasc_inv takes an estimate to the electron pressure and improves
!    it using the temperature and the gas pressure 


      subroutine gasc_inv(t,pg,pe)
      use Param_structure
      use Atomic_data
      parameter (ncontr=28)
      real cmol(91),alfai(ncontr),chi, &
       u0(ncontr),u1(ncontr),u2(ncontr)
      real du0,du1,du2,dcmol(91)
      real pp(10),p(28), tar(1), anear(1), onear(1)
      real g2ar(1), g3ar(1), aar(1), bar(1)
! Interface for saha
      Interface
         Function Saha(npoints, T, Ne, U1, U2, E_ioniz)
           Real, dimension (npoints) :: T, Ne, U1, U2, Saha
           Real :: E_ioniz
           Integer :: npoints
         End Function Saha
      End Interface
!
      tar(1)=t
      onear(1)=1.
      theta=5040./t
      call molecb(theta,cmol,dcmol)
      g4=pe*10.**(cmol(1))
      g5=pe*10.**(cmol(2))
!	ahora calculo los niveles u0,u1,u2 y sus derivadas
6     do i=1,ncontr
         Call Partition_f(i,t,u0(i),u1(i),u2(i),du0,du1,du2)
      End do
      
      ane=pe/bk/t
      anear(1)=ane
      chi=At_ioniz1(1)*eV_to_cgs
      g2ar=1./Saha(1,tar,anear,u0(1),u1(1),chi)   ! p(h+)/p(h)
      g2=g2ar(1)
      If (g2 .le. 1.e-37) g2=0.
      chi=0.754*eV_to_cgs
      g3ar=Saha(1,tar,anear,onear,u0(1),chi)     ! p(h-)/p(h)
      g3=g3ar(1)
      If (g3 .le. 1.e-37) g3=0.
      pp(10)=g3
      g1=0.
      do 1 i=2,ncontr
        chi=At_ioniz1(i)*eV_to_cgs
        aar=1./Saha(1,tar,anear,u0(i),u1(i),chi)
        a=aar(1)
        If (a .le. 1.e-37) a=0.
        chi=At_ioniz2(i)*eV_to_cgs
        bar=1./Saha(1,tar,anear,u1(i),u2(i),chi)
        b=bar(1)
        If (b .le. 1.e-37) b=0.
	c=1.+a*(1.+b) !Nmero total partido por nmero de neutros
        p(i)=(10.**(At_abund(i)-12.))/c   ! p/ph' for neutral he,li, ...
1	g1=g1+p(i)*a*(1.+2.*b)

        pp(2)=p(2)
        pp(3)=p(6)
        pp(4)=p(11)
        pp(5)=p(12)
 
        a=1.+g2+g3
        b=2.*(1.+g2/g5*g4)
        c=g5
        d=g2-g3
        e=g2/g5*g4
        c1=c*b**2+a*d*b-e*a**2
        c2=2.*a*e-d*b+a*b*g1	
        c3=-(e+b*g1)
        f1=0.5*c2/c1
        
! Hctor: Check that f1 won't go to infinity at high temperatures
        
        if (abs(f1).lt.abs(1e3*c3/c1)) then
          f1=-f1+sign(1.,c1)*sqrt(f1**2-c3/c1)
        else
          f1=f1*(-1+sign(1.,c1))-sign(1.,c1)*.5*c3/c1/f1
        end if        

        f5=(1.-a*f1)/b
        f4=e*f5
        f3=g3*f1
        f2=g2*f1
        fe=f2-f3+f4+g1
        If (fe .eq. 0) fe=Max(Abs(f2),Abs(f3),Abs(f4),Abs(g1))/1000.
	phtot=pe/fe

        if(f5.gt.1.e-4) goto 2
          const6=g5/pe*f1**2
          const7=f2-f3+g1

	  do 3 i=1,5
      	       f5=phtot*const6
	       f4=e*f5
	       fe=const7+f4
	       phtot=pe/fe
3	  continue
      
2       pe=pg/(1.+(f1+f2+f3+f4+f5+0.1014)/fe)
        pp(1)=f1   ! p(h)/p(h')
        pp(6)=f2   ! p(h+)/p(h')	
        pp(7)=f5   ! p(h2)/p(h')
        pp(8)=fe   ! pe/p(h')
        pp(9)=pe/(1.38054e-16*t) ! n(e)=pe/kt

      return
End subroutine gasc_inv
