!! Trying to solve the whole Paschen-Back hamiltonian
! Hector: Modifications to interface with LILIA

Subroutine hfs(B_str, Line, s, d, nt, Alow, Blow, Aup, Bup, SpinI)
  Use Param_structure
  Use Line_data_structure
  Type (Line_data) :: Line
  Real :: B_str, Alow, Blow, Aup, Bup, SpinI
  Integer, Parameter :: dd=20 

  logical flag_Paschen_Back
  integer nu,nl,var,i,j,k,k2,iter,ii,idim0
  integer deltam,extu,extl
  integer cuantosu,cuantosl,cual,quel,count,count2,cuall,cualu
  double precision, dimension(dd) :: fu,fl,nmu,nml!,gfu,gfl 
  double precision, dimension(dd) :: Eup0,Elow0
  double precision,dimension (dd,dd) :: mu,ml 
  double precision,dimension(dd,dd) :: Hup,Hlow
  double precision,dimension(dd) :: diag_up,diag_lo,sdiag
  double precision, dimension(3,transitions) :: d,s
  integer, dimension(3) :: nt
  double precision, dimension(3) :: geff
  double precision AS,AP,Eup,Elow,mu0,ju,jl,su,sl,lu,ll,inuc
  double precision BS,BP,C,W,norma,fun3j,fun6j
  double precision  mlo,mup,gu,gl,x,x1,intensidad,sums,str
  common /recuerda2/ nu,nl,nmu,nml,mu,ml,fu,fl,gu,gl,ju,jl,su,sl,lu,ll
!      common /Gfactors/ gfu,gfl
  common /Geff/ geff
  parameter (idim0=1000)

  double precision, dimension(-idim0:idim0) :: sg
  double precision, dimension(0:idim0) :: flog
  common /sgnm/sg
  common /fact/flog

  AS=Alow
  BS=Blow
  AP=Aup
  BP=Bup

  inuc=1.d0*SpinI

  mu0=8.79442d0/(2.d0*3.14159276d0)  ! Bohr's magneton in MHz/G
 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!  We compute the Hfs energies at 0 field  for the two levels
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  If (nu .gt. dd .or. nl .gt. dd) then
     Print *,'nu .or. nl .gt. dd in hfs!!'
     Stop
  End if

  do i=1,nu
     C=fu(i)*(fu(i)+1.d0)-inuc*(inuc+1.d0) - &
          ju*(ju+1.d0)
     W=AP*C/2.d0
         
     if (ju.ne.0.5d0 .and. ju .ne. 0) then
        if (abs(2.d0*inuc*(2.d0*inuc-1.d0)*ju*(2.d0*ju-1.d0)) .lt. 1.e-5) then
           Print *,'Error! Zero denominator in hfs'
           Stop
        End if
        W=W+BP*(.75d0*C*(C+1.d0)-inuc*(inuc+1.d0)*ju*(ju+1.d0))/ &
             (2.d0*inuc*(2.d0*inuc-1.d0)*ju*(2.d0*ju-1.d0))
     endif
     !W is the energy difference of the |F> term respect to the
     ! |(LS)J> level
     Eup0(i)=W
  End do
      
 

  do i=1,nl
     C=fl(i)*(fl(i)+1.d0)-inuc*(inuc+1.d0)- &
          jl*(jl+1.d0)
         
     W=AS*C/2.d0
     if (jl.ne.0.5d0 .and. jl .ne. 0.) then
        if (abs(2.d0*inuc*(2.d0*inuc-1.d0)*ju*(2.d0*ju-1.d0)) .lt. 1.e-5) then
           Print *,'Error! Zero denominator in hfs'
           Stop
        End if
        W=W+BS*(.75d0*C*(C+1.d0)-inuc*(inuc+1.d0)*jl*(jl+1.d0))/ &
             (2.d0*inuc*(2.d0*inuc-1.d0)*jl*(2.d0*jl-1.d0))
     endif
     !W is the energy difference of the |F> term respect to the
     ! |(LS)J> level
     Elow0(i)=W
  End do
      

      
  sums=0.d0
  extu=int(maxval(fu(1:nu)))
      
  extl=int(maxval(fl(1:nl)))
      


! let's go over all the would-be transitions, polarization-ordered
  do i=1,3
!  delta M_F = +1, 0 , -1 so i=1,2,3 respectively
     
        
     deltam=float(2-i)
     count=0
     geff(i)=0.d0
     do j=1,nl       
        signo1=2*(j-1)-1

        do k=1,nml(j)        
           mlo=ml(j,k)
           mup=mlo+deltam

           flag_Paschen_Back=.true.
           Hlow(:,:)=0.d0 !make sure it is initialized 

           do k2=1,nu
              if (k .gt. dd .or. k2 .gt. dd) then
                 print *,'k .or. k2 .gt. dd in hfs'
                 Stop
              End if
              
              var=nmu(k2)
              Hup(:,:)=0.d0!make sure it is initialized 
              if (abs(mup).le.maxval(mu(k2,1:var))) then                  
                 count=count+1
                 if (count .gt. transitions) then
                    Print *,'count .gt. transitions (', &
                         count,' .gt. ',transitions,') in hfs'
                    Stop
                 End if
             
                 !cuantos=min(extu,int(extu-abs(mup)+1)) !the block dimension
                 cuantosu=int(extu-abs(mup)+1)
                 if (abs(mup).lt.(extu-nu+1)) cuantosu=cuantosu-1

                    
                 x=mu0*B_str
                 x=sg(idnint(lu+su+2.d0*ju+inuc-mup))*x
                 x=(2.d0*ju+1.d0)*x
                 x1=sqrt((2.d0*lu+1.d0)*(lu+1.d0)*lu)*&
                      fun6j(lu,lu,1.d0,ju,ju,su)
                 x1=x1+2.d0*sqrt((2.d0*su+1.d0)*(su+1.d0)*su)*&
                      fun6j(su,su,1.d0,ju,ju,lu)

                 do ii=1,cuantosu
                    cual=nu-cuantosu+ii
                    diag_up(ii)=x*fun3j(fu(cual),1.d0,fu(cual),&
                         -mup,0.d0,mup)*fun6j(ju,ju,1.d0,&
                         fu(cual),fu(cual),inuc)*&
                         (2.d0*fu(cual)+1.d0)
                    diag_up(ii)=x1*diag_up(ii)+Eup0(cual)

                    Hup(ii,ii)=1.d0
                       
                    if (ii.gt.1) then
                       quel=cual-1
                       sdiag(ii)=x*fun3j(fu(cual),1.d0,fu(quel),&
                            -mup,0.d0,mup)*fun6j(ju,ju,1.d0,&
                            fu(cual),fu(quel),inuc)*&
                            sqrt((2.d0*fu(cual)+1.d0)*&
                            (2.d0*fu(quel)+1.d0))
                       sdiag(ii)=x1*sdiag(ii)*sg(idnint(fu(cual)+fu(quel)))
                    else
                       sdiag(ii)=1.d0
                    endif
                 End do
                 
                 do ii=cuantosu+1,dd 
                    Hup(ii,ii)=1.d0
                 End do
                    
                   
                 if (cuantosu.gt.1) then
                    call tqli(diag_up,sdiag,cuantosu,dd,Hup)
                 endif
                    
                 do ii=1,cuantosu
                    norma=sqrt(sum(Hup(1:cuantosu,ii)**2))
                    Hup(1:cuantosu,ii)=Hup(1:cuantosu,ii)/norma
                 End do

                 cualu=k2-nu+cuantosu !For intensity computations
!                 if (cualu .lt. 1) cualu=1. ! debug. HECTOR
                 Eup=diag_up(cualu) ! We are looking for 
                                    !the k2-th eigenvalue

!!!!!!   Let's calcultate the lower level eigenvalues

                 if (flag_Paschen_Back) then

                    ! cuantos=min(extl,int(extl-abs(mlo)+1)) 
                              !the block dimension
                    cuantosl=int(extl-abs(mlo)+1)
                    if (abs(mlo).lt.(extl-nl+1)) cuantosl=cuantosl-1

                    x=mu0*B_str
                    x=sg(idnint(ll+sl+2.d0*jl+inuc-mlo))*x
                    x=(2.d0*jl+1.d0)*x
                    x1=sqrt((2.d0*ll+1.d0)*(ll+1.d0)*ll)*&
                         fun6j(ll,ll,1.d0,jl,jl,sl)
                    x1=x1+2.d0*sqrt((2.d0*sl+1.d0)*(sl+1.d0)*sl)*&
                         fun6j(sl,sl,1.d0,jl,jl,ll)

                    do ii=1,cuantosl
                       cual=nl-cuantosl+ii 
!                       if (cual .lt. 1) cual=1. ! debug. HECTOR
                       diag_lo(ii)=x*fun3j(fl(cual),1.d0,fl(cual),&
                            -mlo,0.d0,mlo)* fun6j(jl,jl,1.d0,fl(cual),&
                            fl(cual),inuc)*(2.d0*fl(cual)+1.d0)

                       diag_lo(ii)=x1*diag_lo(ii)+Elow0(cual)

                       Hlow(ii,ii)=1.d0
                       
                       if (ii.gt.1) then
                          quel=cual-1
!                       if (quel .lt. 1) quel=1. ! debug. HECTOR
                       sdiag(ii)=x*fun3j(fl(cual),1.d0,fl(quel),&
                               -mlo,0.d0,mlo)*fun6j(jl,jl,1.d0,&
                               fl(cual),fl(quel),inuc)*&
                               sqrt((2.d0*fl(cual)+1.d0)*&
                               (2.d0*fl(quel)+1.d0))
                          sdiag(ii)=x1*sdiag(ii)*sg(idnint(fl(cual)+fl(quel)))
                       else
                          sdiag(ii)=1.d0
                       endif
                    End do
                 
                    do ii=cuantosl+1,dd 
                       Hlow(ii,ii)=1.d0
                    End do
                    if (cuantosl.gt.1) then
                       call tqli(diag_lo,sdiag,cuantosl,dd,Hlow)
                    endif
                    do ii=1,cuantosl
                       norma=sqrt(sum(Hlow(1:cuantosl,ii)**2))
                       Hlow(1:cuantosl,ii)=Hlow(1:cuantosl,ii)/norma
                    End do

                    flag_Paschen_Back =.false. 
                 endif !Only 1 diagonalization of lower level needed

                 cuall=j-nl+cuantosl !For intensity computations
!                 if (cuall .lt. 1) cuall=1. ! debug. HECTOR
                 if (cuall .gt. dd) then
                    Print *,'cuall .gt. dd (',cuall,' .gt. ',dd,') in hfs'
                    print *,j,nl,cuantosl
                    Stop
                 End if
                 Elow=diag_lo(cuall)  ! We are looking for
                                      ! the j-th eigenvalue
                    
 !                   print*,diag_lo(1:2),Elow0(1:2)
                 s(i,count)=intensidad(Line,Hlow,Hup,mlo,mup,&
                     fl,fu,nl,nu,extu,extl,cualu,cuall, SpinI)

!                    x=(2.997924582d12/linea%long_onda)+(Eup-Elow)
!                    d(i,count)=1000.d0*(2.997924583d12/x-linea%long_onda) !in mA

                 d(i,count)=-1.d-9*((Eup-Elow))*&
                      Line%Wlength**2/2.997924583d0

!                      linea%long_onda**2/2.997924583d0
                    

!                    d(i,count)=Eup !For testing purposes
              endif
           End do
        End do
     End do
     sums=sums+ sum(s(i,1:count))
     nt(i)=count
     If (nt(i) .gt. transitions) then
        Print *,'Too many transitions (',nt(i),')'
        Print *,'The Hyperfine Structure routines are prepared to work with ',&
             transitions,'. STOP in hfs'
        Stop
     End if
  End do

!      d=3.0e12/(2.*3.14159276*d) ! in A

  s=s/sums

end subroutine hfs


double precision function intensidad(Line,Hlow,Hup,ml,mu,fl,fu &
     ,nl,nu,extu,extl,cual,quel, SpinI)
  Use Param_structure
  Use Line_data_structure
  Type (Line_data) :: Line
  Integer, Parameter :: dd=20
  Real :: SpinI
integer i,j,idim0,pu,pl
integer cuantosu,cuantosl,extu,extl,cual,quel,cons
double precision f1,f2,str,ml,mu,fun3j,fun6j
double precision inuc,ju,jl
double precision, dimension(dd,dd) :: Hlow,Hup
double precision, dimension(dd) :: fu,fl
parameter (idim0=1000) 

double precision, dimension(-idim0:idim0) :: sg
double precision, dimension(0:idim0) :: flog
common /sgnm/sg
common /fact/flog

!cuantosu=min(extu,int(extu-abs(mu)+1))
cuantosu=int(extu-abs(mu)+1)
if (abs(mu).lt.(extu-nu+1)) cuantosu=cuantosu-1
!cuantosl=min(extl,int(extl-abs(ml)+1))
cuantosl=int(extl-abs(ml)+1)
if (abs(ml).lt.(extl-nl+1)) cuantosl=cuantosl-1

inuc=1.d0*SpinI
ju=1.d0*Line%J_up
jl=1.d0*Line%J_low
!print*,'up:',Hup(1:2,1:2),cual
!print*,'down:',Hlow(1:2,1:2),quel

intensidad=0.d0
do i=1,cuantosu
   if (Hup(i,cual).ne.0.) then
      f1=fu(nu-cuantosu+i) 
     
      do j=1,cuantosl
         if (Hlow(j,quel).ne.0.) then
            cons=nl-cuantosl+j
!            if (cons .lt. 1) cons=1 ! debug. HECTOR
            f2=fl(cons) 
            intensidad=intensidad+sg(idnint(-mu+f1+f2+inuc+jl))*&
                 sqrt((2.d0*f1+1.d0)*(2.d0*f2+1.d0)*(2.d0*ju+1.d0))*&
                 fun6j(f1,f2,1.d0,jl,ju,inuc)*&
!                 fun6j(ju,jl,1.d0,f2,f1,inuc)*&
                 fun3j(f1,1.d0,f2,-mu,mu-ml,ml)*(Hup(i,cual)*Hlow(j,quel))
         endif
      End do
   endif
End do
intensidad=intensidad**2
end function intensidad

subroutine hfs_int(Line, SpinI)
  Use Param_structure
  Use Line_data_structure
  Type (Line_data) :: Line
  Real :: OAM, SpinI
  Integer, Parameter :: dd=20

  integer nu,nl,var,i,j,k,k2
  integer mlo,mup,deltam
  double precision su,lu,ju,gu,sl,gl,ll,jl,str
  double precision, dimension(dd) :: fu,fl,nmu,nml!,gfu,gfl 
  double precision,dimension (dd,dd) :: mu,ml 
  double precision, dimension(3,transitions) :: s
  integer, dimension(3) :: nt
  double precision, dimension(3) :: geff
  common /recuerda2/ nu,nl,nmu,nml,mu,ml,fu,fl,gu,gl,ju,jl,su,sl,lu,ll
!      common /Gfactors/ gfu,gfl
!****** Upper level   
  su=(Line%Mult_low-1.)/2.
  Call Extract_OAM(Line%Desig_up, OAM)
  lu=OAM
  ju=Line%J_up
  gu = 0.
  if(ju.ne.0.) gu = 1.5+(su*(su+1.)-lu*(lu+1.))/(2.*ju*(ju+1.))

  if (Line%J_up .gt. SpinI) then 
     nu=2*SpinI+1
  else 
     nu=2*Line%J_up+1        
  endif
      
  do i=1,nu
     fu(i)=i-1.+abs(-Line%J_up+SpinI)
     nmu(i)=2*fu(i)+1
!        gfu(i)=gu*(fu(i)*(fu(i)+1.)+ju*(ju+1.)-linea%Inuclear*&
!             (linea%Inuclear+1.))/(2.*fu(i)*(fu(i)+1.))
     do k=1,nmu(i)
        mu(i,k)=-fu(i)-1.+k
     End do
  End do
      
!****** Lower level   
  sl=(Line%Mult_low-1.)/2.
  gl = 0.
  Call Extract_OAM(Line%Desig_low, OAM)
  ll=OAM
  jl=Line%J_low
      
  if(jl.ne.0.) gl = 1.5+(sl*(sl+1.)-ll*(ll+1.))/(2.*jl*(jl+1.))

  if (Line%J_low .gt. SpinI) then 
     nl=2*SpinI+1
  else 
     nl=2*Line%J_low+1        
  endif
      
  do i=1,nl
     fl(i)=i-1.+abs(-Line%J_low+SpinI)
     nml(i)=2*fl(i)+1
!         gfl(i)=gl*(fl(i)*(fl(i)+1.)+jl*(jl+1.)-linea%Inuclear*&
!             (linea%Inuclear+1.))/(2.*fl(i)*(fl(i)+1.))
     do k=1,nml(i)
        ml(i,k)=-fl(i)-1.+k
     End do
  End do
      
  sums=0.

  return
  
end subroutine hfs_int



  
integer function signe (a)
  
  real a

  signe = 1
  if (a.lt.0) signe=-1

end function signe
