  Use Param_structure
  Use Model_structure
  Use Line_data_structure
!
  Implicit None
  Type (Parameters) :: Params
  Type (Line_data), dimension (1) :: Line  ! Must match params%n_lines
  Type (Region_data), dimension (1) :: Region! Must match params%n_regions
  Type (Model) :: Atmo
  Logical :: Error, Hydro
  Integer :: idepth
!
  Real, Dimension (400) :: Syn_profile ! Must match params%n_data
!
! Things you need to define
! Model atmo
  Params%n_points=40 ! z-points in model atmo
  Call Allocate_model(Atmo, Params%n_points)
  ! Note: Model must be arranged in decreasing order of tau_500, so the first
  !   point is the top of the atmosphere
  Atmo%el_p(1)=6e-2 ! Boundary condition for electron pressure at the top
  Atmo%v_mac=0.
  Atmo%stray=0.
  Atmo%expansion=0.
  Do idepth=1,40
     Atmo%ltau_500(idepth)=idepth/5.-2 ! Log(tau_5000)
     Atmo%temp(idepth)=8000-idepth*100 ! Linearly decreasing temperature
     Atmo%v_mic(idepth)=0. ! Microturbulence (cm/s)
     Atmo%v_los(idepth)=0. ! l.o.s. velocity (cm/s)
     Atmo%b_str(idepth)=300. ! Field strength (G)
     Atmo%b_inc(idepth)=180. ! Field inclination (deg, 0=toward observer)
     Atmo%b_azi(idepth)=0. ! Field azimuth (deg)
  End do

! Spectral regions and lines
  Params%n_regions=1 ! Spectral regions
  Params%n_lines=1 ! Spectral lines
  Params%n_data=400 ! 4 times Number of wavelengths (for I,Q,U,V)
  Region(1)%first_wlength=6300
  Region(1)%wave_step=0.020
  Region(1)%nwavelengths=100
  Region(1)%Macro_enh=1.
  Region(1)%layer=1
  Line(1)%Wlength=6301.501
  Line(1)%Mult_low=5
  Line(1)%Mult_up=5
  Line(1)%Desig_low='P'
  Line(1)%Desig_up='D'
  Line(1)%J_low=2.0
  Line(1)%J_up=2.0
  Line(1)%VDW_enh=1.
  Line(1)%Energy_low=3.65
  Line(1)%loggf=-0.718
  Line(1)%Atomic_number=26
  Line(1)%Ion_stage=1
  Line(1)%Atomic_weight=56
  Line(1)%collisions=1
! Other
  Params%printout=0 ! Verbosity level
  Params%speed=2 ! Some speed optimizations
  Params%reference_cont=1 ! How to normalize emergent intensity
  Params%Skip_lambda=0 ! Leave it at 0
  Params%heliocentric=1. ! Heliocentric angle
  Params%Update_opac=1 ! How many Angstroms before recomputing background opac
  Hydro=.True. ! Set model in hydrostatic equilibrium
  
!
! Compute profiles
  Call Forward(Params, Line, Region, Atmo, Syn_profile, Hydro, Error)

End Program
