! Compute LTE level populations using information in the ATOM file
! Note: This routine requires a valid Atom%ne array, so a call to
! hydrostatic before this is a good idea.
!
Subroutine LTE_pop_2(NLTE, Atom)
!
  Use Param_structure
  Use NLTE_Atom_structure
  Use NLTE_vars
  Use Model_structure
  Use Atomic_data
  Implicit None
  Type (NLTE_Atom) :: Atom
  Type (NLTE_variables) :: NLTE
  Real, dimension (NLTE%NDEP) :: N_e, U0, U1, U2, U_ion
  Real, dimension (NLTE%NDEP) :: N_ion, N0_N1, N1_N2, N_tot, N0, N1, N2
  Integer :: index, npoints, idepth, iline, Atomic_number, ilev
  Real :: du0, du1, du2, E_ioniz1, E_ioniz2, E_exc_i, E_exc_j, Abund_mass
  Real :: E_ion, E_exc, E_base
  Logical, Save :: Warning=.FALSE.
! Interface for the function Saha
  Interface
     Function Saha(npoints, T, Ne, U1, U2, E_ioniz)
       Real, dimension (npoints) :: T, Ne, U1, U2, Saha
       Real :: E_ioniz
       Integer :: npoints
     End Function Saha
  End interface
!
  npoints=NLTE%NDEP
  Atomic_number=Atom%Z
  Abund_mass=Atom%AWgt*(10.**(Atom%Abund-12.))/ &
       Sum(At_weight(1:N_elements)*(10.**(At_abund(1:N_elements)-12.)))
  N_tot(1:npoints)=Abund_mass*NLTE%Atmo%rho(1:npoints)/Atom%Awgt/mass_pr
  N_e(1:npoints)=NLTE%Atmo%ne
  Do idepth=1, npoints
     Call Partition_f(Atomic_number, NLTE%Atmo%Temp(idepth), &
          U0(idepth), U1(idepth), U2(idepth), du0, du1, du2)
  End do
  E_ioniz1=At_ioniz1(Atomic_number)*eV_to_cgs
  E_ioniz2=At_ioniz2(Atomic_number)*eV_to_cgs
!
! Saha ionization equations
!
  N0_N1(1:npoints)=Saha(npoints, NLTE%Atmo%Temp, N_e, U0, U1, E_ioniz1)
  N1_N2(1:npoints)=Saha(npoints, NLTE%Atmo%Temp, N_e, U1, U2, E_ioniz2)
  N1(1:npoints)=N_tot(1:npoints)/  &
       (1.+N0_N1(1:npoints)+1./N1_N2(1:npoints)) ! N_1 (cm^-3)
  N0(1:npoints)=N0_N1(1:npoints)*N1(1:npoints) ! N_0 (cm^-3)
  N2(1:npoints)=N1(1:npoints)/N1_N2(1:npoints) ! N_2 (cm^-3)
!
  Do ilev=1, Atom%NK
     If (Atom%ion(ilev) .eq. 1) then
        N_ion(1:npoints)=N0(1:npoints)
        U_ion(1:npoints)=U0(1:npoints)
        E_ion=0
     Else if (Atom%ion(ilev) .eq. 2) then
        N_ion(1:npoints)=N1(1:npoints)
        U_ion(1:npoints)=U1(1:npoints)
        E_ion=E_ioniz1
     Else if (Atom%ion(ilev) .ge. 3) then
        N_ion(1:npoints)=N2(1:npoints)
        U_ion(1:npoints)=U2(1:npoints)
        E_ion=E_ioniz1+E_ioniz2
     End if
     If (Atom%ion_stage .eq. 2) then
        E_base=At_ioniz1(Atomic_number)
     Else if (Atom%ion_stage .eq. 3) then
        E_base=At_ioniz2(Atomic_number)
     End if
     If (Atom%ion(ilev) .gt. 3 .and. .not. Warning) then
        Print *,'Ionization stage gt 3 in LTE_pop_2'
        Stop
     End if
     E_exc=( (Atom%eV(ilev)+E_base)*eV_to_cgs - E_ion)
     If (E_exc .lt. 0) then 
        If (E_exc .gt. -1e-18) then ! Numerical accuracy
           E_exc=0.
        Else
           Print *,'Negative excitation potential in LTE_pop_2'
           Stop
        End if
     End if
     NLTE%Nstar(ilev,1:npoints)=N_ion(1:npoints)*Atom%g(ilev)* & ! Boltzmann
          exp(-E_exc/bk/NLTE%Atmo%Temp(1:npoints))/U_ion(1:npoints)
  End do
! Check that the sum of all levels doesn't exceed totn
  Do idepth=1, NLTE%NDEP
     If (Sum(NLTE%Nstar(:,idepth))/N_Tot(idepth) .gt. 1.1) then
        Print *,'Error in forward/NLTE/ltepop2.f90'
        Print *,'At depth point:',idepth
        Print *,'The total population of all levels is ', &
             Sum(NLTE%Nstar(:,idepth))
        Print *,'According to abundance data it should be .lt. ',N_Tot(idepth)
        Print *,'This might indicate an error in the G values in the ATOM file'
        Stop
     End if
  End do
!
  Return
End Subroutine LTE_pop_2
