! Interface with Arturo's hyperfine structure routines
!
Subroutine HFInterface(Line, B_str, np, nl, nr, &
             dlp, dll, dlr, sp, sl, sr, Alow, Blow, Aup, Bup, SpinI)
  Use Param_structure
  Use Line_data_structure
  Implicit None
  Type (Line_data) :: Line
  Real, Parameter :: Threshold = 1.e-3
  Real :: B_str, dlo, Maximum
  Real :: Alow, Blow, Aup, Bup, SpinI
  Integer :: np, nr, nl, i
  Real, Dimension (transitions) :: dlp, dll, dlr, sp, sl, sr
      
  integer flag_humlicek, flag_breit,j,k,iter,ll,nu
  double precision, dimension(3) :: hgen,fgen
  integer, dimension(3):: nt
  double precision, dimension(3, transitions) :: d,s
  double precision vshift,dmag,h,f,hp,fvp,hb,fvb,hr,fvr

  B_str=Max(B_str,1.e-2) ! Make sure field is .gt. 0
  Call hfs_int(Line, SpinI) 
  Call hfs(B_str, Line, s, d, nt, Alow, Blow, Aup, Bup, SpinI)

  np=nt(2)
  nl=nt(3)
  nr=nt(1)

! The sign comes from different interpretation of the shift in mvoigt and
! hfs. In mvoigtc a positive dlp,dll or dlr means that this component will
! be shifted to shorter wavelengths

  dlp(:)=-d(2,:)/B_str*1.e-11 ! mA to cm/G
  dll(:)=-d(3,:)/B_str*1.e-11 ! mA to cm/G
  dlr(:)=-d(1,:)/B_str*1.e-11 ! mA to cm/G

  sp(:)=s(2,:)*3. ! Arturo's amplitudes are normalized so that the sum of all
  sl(:)=s(3,:)*3. ! Pi components is 1. I have it so that the sum of all Pi
  sr(:)=s(1,:)*3. ! plus all Sigma-left plus all Sigma-right is 1.

! Remove negligible components

  Maximum=MaxVal(Abs(sp(1:np)))
  Do i=1, np
     If (Abs(sp(i)) .lt. Threshold*Maximum) then
        sp(i:np-1)=sp(i+1:np)
        dlp(i:np-1)=dlp(i+1:np)
        np=np-1
     End if
  End do
  Maximum=MaxVal(Abs(sl(1:nl)))
  Do i=1, nl
     If (Abs(sl(i)) .lt. Threshold*Maximum) then
        sl(i:nl-1)=sl(i+1:nl)
        dll(i:nl-1)=dll(i+1:nl)
        nl=nl-1
     End if
  End do
  Maximum=MaxVal(Abs(sr(1:nr)))
  Do i=1, nr
     If (Abs(sr(i)) .lt. Threshold*Maximum) then
        sr(i:nr-1)=sr(i+1:nr)
        dlr(i:nr-1)=dlr(i+1:nr)
        nr=nr-1
     End if
  End do

End Subroutine HFInterface
