Module Param_structure
!
! Hardware-dependent parameters
!
! Number of bytes in a default real number. Note that some compilers,
!  such as ifort interpret this as the number of r4 in a record. In 
!  such cases RealBytes must be set to 1
! Integer, Parameter :: RealBytes=4
  Integer, Parameter :: RealBytes=2 ! Number of bytes in a default real number
!
! Run-dependent parameters
!
  Type Parameters
     Integer :: n_data, n_free_parameters, n_variables
     Integer :: n_points, n_lines, n_regions, ncycles
     Integer :: formal_solution, printout
     Integer :: nx, ny, def_abund, speed, always_compute_deriv
     Integer :: recompute_deriv, reference_cont
     Integer :: Skip_lambda
     Real :: SVD_threshold, heliocentric, helioazim
     Real :: Noise, Regularization, Update_opac
     Real, Dimension (:), pointer :: Stray_prof ! For the stray light
  End Type Parameters
! Maximum allowed Zeeman components per spectral line
  Integer, Parameter :: transitions=1000 
! Solar constants
! c.g.s. units
  Real :: Gravity=2.7414e+4 ! cm/s^2
  Real, Parameter :: Solar_Gravity = 2.7414e+4 ! Surface gravity (cm/s^2)
! Physical constants
! c.g.s. units
  Real, Parameter :: cc = 2.99792458D10, hh = 6.62618D-27, ee = 1.602189D-12 
  Real, Parameter :: mass_el = 9.10953D-28, mass_pr = 1.672648D-24
  Real, Parameter :: bk = 1.38066D-16, Avog=6.022045D23
! Conversion factor: eV to erg (g cm^2 / s^2). 
! Note that 1 V = 10^7 (g cm^2)/(C s^2)
  Real, Parameter :: eV_to_cgs = ee
! Mathematical constants
  Real, Parameter :: Pi = 3.14159265389793D0
  Real, Parameter :: SqrtPi = 1.77245390415191650391 ! Sqrt(Pi)
  Real, Parameter :: Piis = 0.56418956659890107D0 ! Inverse of the Sqrt of Pi
End Module Param_structure
